"""Main J.A.R.V.I.S. package."""
import logging

from dis_snek import Intents, Snake, listen
from mongoengine import connect

# from jarvis import logo  # noqa: F401
from jarvis import tasks, utils
from jarvis.config import get_config
from jarvis.events import member, message

jconfig = get_config()

logger = logging.getLogger("discord")
logger.setLevel(logging.getLevelName(jconfig.log_level))
file_handler = logging.FileHandler(filename="jarvis.log", encoding="UTF-8", mode="w")
file_handler.setFormatter(logging.Formatter("[%(asctime)s][%(levelname)s][%(name)s] %(message)s"))
logger.addHandler(file_handler)

intents = Intents.DEFAULT
intents.members = True
restart_ctx = None


jarvis = Snake(intents=intents, default_prefix="!", sync_interactions=jconfig.sync)

__version__ = "2.0.0a0"


@listen()
async def on_ready() -> None:
    """Lepton on_ready override."""
    global restart_ctx
    print("    Logged in as {0.user}".format(jarvis))  # noqa: T001
    print("    Connected to {} guild(s)".format(len(jarvis.guilds)))  # noqa: T001


@listen()
async def on_startup() -> None:
    """Lepton on_startup override."""
    tasks.init()


def run() -> None:
    """Run J.A.R.V.I.S."""
    connect(
        db="ctc2",
        alias="ctc2",
        authentication_source="admin",
        **jconfig.mongo["connect"],
    )
    connect(
        db=jconfig.mongo["database"],
        alias="main",
        authentication_source="admin",
        **jconfig.mongo["connect"],
    )
    jconfig.get_db_config()

    for extension in utils.get_extensions():
        jarvis.load_extension(extension)

    print(  # noqa: T001
        "    https://discord.com/api/oauth2/authorize?client_id="
        "{}&permissions=8&scope=bot%20applications.commands".format(jconfig.client_id)
    )

    jarvis.max_messages = jconfig.max_messages

    # Add event listeners
    if jconfig.events:
        _ = [
            member.MemberEventHandler(jarvis),
            message.MessageEventHandler(jarvis),
        ]
    jarvis.start(jconfig.token)
