"""J.A.R.V.I.S. twitter background task handler."""
import logging
from asyncio import to_thread
from datetime import datetime, timedelta

import tweepy
from dis_snek.ext.tasks.task import Task
from dis_snek.ext.tasks.triggers import IntervalTrigger

import jarvis
from jarvis.config import get_config
from jarvis.db.models import Twitter

logger = logging.getLogger("jarvis")

__auth = tweepy.AppAuthHandler(
    get_config().twitter["consumer_key"], get_config().twitter["consumer_secret"]
)
__api = tweepy.API(__auth)


async def _tweets() -> None:
    """J.A.R.V.I.S. twitter blocking task."""
    guild_cache = {}
    channel_cache = {}
    twitters = Twitter.objects(active=True)
    for twitter in twitters:
        try:
            if not twitter.twitter_id or not twitter.last_sync:
                user = __api.get_user(screen_name=twitter.handle)
                twitter.twitter_id = user.id
                twitter.handle = user.screen_name
                twitter.last_sync = datetime.now()

            if twitter.last_sync + timedelta(hours=1) <= datetime.now():
                user = __api.get_user(id=twitter.twitter_id)
                twitter.handle = user.screen_name
                twitter.last_sync = datetime.now()

            if tweets := __api.user_timeline(id=twitter.twitter_id):
                guild_id = twitter.guild
                channel_id = twitter.channel
                tweets = sorted(tweets, key=lambda x: x.id)
                if guild_id not in guild_cache:
                    guild_cache[guild_id] = await jarvis.jarvis.get_guild(guild_id)
                guild = guild_cache[twitter.guild]
                if channel_id not in channel_cache:
                    channel_cache[channel_id] = await guild.fetch_channel(channel_id)
                channel = channel_cache[channel_id]
                for tweet in tweets:
                    retweet = "retweeted_status" in tweet.__dict__
                    if retweet and not twitter.retweets:
                        continue
                    timestamp = int(tweet.created_at.timestamp())
                    url = f"https://twitter.com/{twitter.handle}/status/{tweet.id}"
                    verb = "re" if retweet else ""
                    await channel.send(
                        f"`@{twitter.handle}` {verb}tweeted this at <t:{timestamp}:f>: {url}"
                    )
                newest = max(tweets, key=lambda x: x.id)
                twitter.last_tweet = newest.id
                twitter.save()
        except Exception as e:
            logger.error(f"Error with tweets: {e}")


@Task.create(trigger=IntervalTrigger(minutes=1))
async def tweets() -> None:
    """J.A.R.V.I.S. twitter background task."""
    await to_thread(_tweets)
