"""J.A.R.V.I.S. unwarn background task handler."""
from asyncio import to_thread
from datetime import datetime, timedelta

from dis_snek.ext.tasks.task import Task
from dis_snek.ext.tasks.triggers import IntervalTrigger

from jarvis.db.models import Warning


async def _unwarn() -> None:
    """J.A.R.V.I.S. unwarn blocking task."""
    warns = Warning.objects(active=True)
    for warn in warns:
        if warn.created_at + timedelta(hours=warn.duration) < datetime.utcnow():
            warn.active = False
            warn.save()


@Task.create(IntervalTrigger(hours=1))
async def unwarn() -> None:
    """J.A.R.V.I.S. unwarn background task."""
    await to_thread(_unwarn)
